package executor

import (
	"fmt"
	"github.com/33cn/chain33/client"
	dbm "github.com/33cn/chain33/common/db"
	tab "github.com/33cn/chain33/common/db/table"
	"github.com/33cn/chain33/types"
	brokertypes "github.com/33cn/plugin/plugin/dapp/broker/types"
	storagetypes "github.com/33cn/plugin/plugin/dapp/storage/types"
)

//BrokerDB ...
type BrokerDB struct {
	api       client.QueueProtocolAPI
	db        dbm.KV
	localdb   dbm.KV
	txhash    []byte
	fromaddr  string
	blocktime int64
	height    int64
	index     int
}

func newBrokerDB(b *broker, tx *types.Transaction, index int) *BrokerDB {
	hash := tx.Hash()
	fromaddr := tx.From()
	return &BrokerDB{b.GetAPI(), b.GetStateDB(), b.GetLocalDB(), hash, fromaddr,
		b.GetBlockTime(), b.GetHeight(), index}
}

////GetKVSet ...
//func (b *BrokerDB) GetKVSet() (kvset []*types.KeyValue) {
//	kvset = append(kvset, &types.KeyValue{Key: Key(common.ToHex(s.txhash)), Value: types.Encode(payload)})
//	return kvset
//}
func(b *BrokerDB)register(payload *brokertypes.Register )(*types.Receipt, error){

	return nil,nil
}

func(b *BrokerDB)audit(payload *brokertypes.Audit )(*types.Receipt, error){

	return nil,nil
}

func(b *BrokerDB)updateEventStatus(payload *brokertypes.UpdateEventStatus)(*types.Receipt, error){
	receipt := &types.Receipt{Ty: types.ExecOk}
	log :=&brokertypes.ReceiptBrokerLog{Value:&brokertypes.ReceiptBrokerLog_UpdateEventStatus{payload},Ty:brokertypes.TyEmitInterchainEventAction}
	receiptlog := &types.ReceiptLog{Ty:brokertypes.TyUpdateEventStatusLog,Log:types.Encode(log)}
	receipt.Logs=append(receipt.Logs,receiptlog)
	return receipt,nil
}
func(b *BrokerDB)emitInterchainEvent(payload *brokertypes.InterchainEvent )(*types.Receipt, error){
	receipt := &types.Receipt{Ty: types.ExecOk}
	//发布跨链事件  分为1.信息跨链 2.代币跨链，代币跨链需要在这步将所需金额冻结在broker下面
	if payload.ReqType == brokertypes.Req_Type_Storage {
		//校验存在交易是否存在
		params :=&storagetypes.QueryStorage{TxHash:payload.Args[0]}
		_,err:=b.api.Query(storagetypes.StorageX,payload.GetFunc(),params)
		if err != nil {
			return nil,brokertypes.ErrBrokerStorageTx
		}
       log :=&brokertypes.ReceiptBrokerLog{Value:&brokertypes.ReceiptBrokerLog_EmitInterchainEvent{payload},Ty:brokertypes.TyEmitInterchainEventAction}
       receiptlog := &types.ReceiptLog{Ty:brokertypes.TyEmitInterchainEventLog,Log:types.Encode(log)}
       receipt.Logs=append(receipt.Logs,receiptlog)
	}

	if payload.ReqType == brokertypes.Req_Type_Coins {
		//TODO 跨链转账

	}
	return receipt,nil
}



//根据状态遍历时间列表（可通过定时轮循这个接口，获取跨链时间）
func findInterChainEventListByStatus(localdb dbm.KV,status, direction int32, primaryKey string) (*brokertypes.InterchainEventList, error) {
	table := NewInterchainEventTable(localdb)
	prefix := []byte(fmt.Sprintf("%d",status))

	var rows []*tab.Row
	var err error
	if primaryKey == "" { //第一次查询,默认展示最新得成交记录
		rows, err = table.ListIndex("status", prefix, nil, brokertypes.PageSize, direction)
	} else {
		rows, err = table.ListIndex("status", prefix, []byte(primaryKey), brokertypes.PageSize, direction)
	}
	if err != nil {
		elog.Error("findInterChainEventListByStatus.",  "err", err.Error())
		return nil, err
	}
	var eventList brokertypes.InterchainEventList
	for _, row := range rows {
		event := row.Data.(*brokertypes.InterchainEvent)
		eventList.List = append(eventList.List, event)
	}
	//设置主键索引
	if len(rows) == int(brokertypes.PageSize) {
		eventList.PrimaryKey = string(rows[len(rows)-1].Primary)
	}
	return &eventList, nil
}
//根据index查询event
func findInterChainEventByIndex(localdb dbm.KV,index uint64)(*brokertypes.InterchainEvent,error){
	table :=NewInterchainEventTable(localdb)
	primaryKey := []byte(fmt.Sprintf("%018d", index))
	row,err:=table.GetData(primaryKey)
	if err !=nil {
		return nil,err
	}
	return row.Data.(*brokertypes.InterchainEvent),nil
}